/*
 * Transform.java
 *
 * Created on April 5, 2004, 12:34 PM
 */

package com.milowski.jaxp;

import java.util.*;
import java.net.*;
import javax.xml.transform.*;
import javax.xml.transform.stream.*;
import javax.xml.transform.dom.*;

/**
 *
 * @author  alex
 */
public class Transform {

   TransformerFactory tfactory;
   List transforms;
   
   /** Creates a new instance of Transform */
   public Transform() {
      transforms = new ArrayList();
      tfactory = TransformerFactory.newInstance();
   }
   
   public void addTransformation(URL location)
      throws TransformerConfigurationException
   {
      transforms.add(
         tfactory.newTransformer(new StreamSource(location.toString()))
      );
   }
   
   public void addTransformation(String location) 
      throws TransformerConfigurationException
   {
      transforms.add(
         tfactory.newTransformer(new StreamSource(location))
      );
   }
   
   public void transform(String inLocation,String outLocation) 
      throws TransformerException
   {
      // If we have nothing to do, return
      if (transforms.size()==0) {
         return;         
      }
      
      // Setup the input and output
      StreamSource in = new StreamSource(inLocation);
      StreamResult out = new StreamResult(outLocation);
      
      // If we only have on transformation, just do it without DOM
      if (transforms.size()==1) {
         Transformer xform = (Transformer)transforms.get(0);
         xform.transform(in,out);
      } else {
         
         // Otherwise, transform each step with a DOM between
         int len = transforms.size();
         DOMResult between = null;
         for (int i=0; i<len; i++) {
            Transformer xform = (Transformer)transforms.get(i);
            Source start = i==0 ? (Source)in : (Source)new DOMSource(between.getNode());
            if ((i+1)==len) {
               xform.transform(start,out);
            } else {
               between = new DOMResult();
               xform.transform(start,between);
            }
         }
      }
         
   }
   
   public static void main(String [] args) {
      if (args.length<3) {
         System.err.println("Usage: Transform in.xml stylesheet1 { stylesheet2 ... } out.xml");
         System.exit(1);
      }

      // Setup the app
      Transform app = new Transform();
      
      // Get the in and out file
      String in = args[0];
      String out = args[args.length-1];
      
      try {
         
         // Add each transformation
         for (int i=1; i<(args.length-1); i++) {
            app.addTransformation(args[i]);
         }
         
         // Transform the document
         app.transform(in,out);
         
         System.exit(0);
      } catch (TransformerConfigurationException ex) {
         System.err.println(ex.getMessage());
      } catch (TransformerException ex) {
         System.err.println(ex.getMessage());
      } catch (Exception ex){
         ex.printStackTrace();
      }
      System.exit(1);
      
   }
   
}
